/***********
* ztouch.c *
***********/

/* $Id: ztouch.c,v 0.6 1995/03/27 21:54:19 sharl Exp masuda $ */

#include "sys.h"
#include "getopt.h"
#include "revision.h"

typedef char HEAD[10];

char *progname;			/* it's own name */
static time_t time_stamp;	/* original time stamp */
static char *origname;		/* original file name */

static int help;		/* help */
static int noname;		/* do not restore the original name */
static int print;		/* print info only */
static int verbose;		/* verbose mode */
static int version;		/* print version */

struct option long_opt[] =
{
    {"help", no_argument, &help, FALSE},
    {"no-name", no_argument, &noname, FALSE},
    {"print", no_argument, &print, FALSE},
    {"verbose", no_argument, &verbose, FALSE},
    {"version", no_argument, &version, FALSE},
    {0, 0, 0, 0}
};

char *os_name[] =
{
    "MS-DOS",			/* 00 */
    "Amiga",			/* 01 */
    "VAX/VMS",			/* 02 */
    "UNIX",			/* 03 */
    "unknown",			/* 04 */
    "ATARI ST",			/* 05 */
    "OS/2",			/* 06 */
    "Mac",			/* 07 */
    "unknown",			/* 08 */
    "unknown",			/* 09 */
    "TOPS20",			/* 0a */
    "WIN-NT",			/* 0b */
    "unknown",			/* 0c */
    "unknown",			/* 0d */
    "unknown",			/* 0e */
    "PRIMOS",			/* 0f */
};

static BOOL 
ident_gzip (name)
char *name;
{
    FILE *fp;
    HEAD head;
    BOOL retcode = FALSE;

    if ((fp = fopen(name, "rb")) == NULL) {
	if (verbose)
	    fprintf(stderr, "%s: %s: %s\n", progname, name, strerror(errno));
	return FALSE;
    }
    origname = NULL;

    if (fread(head, sizeof(HEAD), 1, fp) != 1) {
	if (verbose)
	    fprintf(stderr, "%s: %s: unexpected end of file\n", progname, name);
	retcode = FALSE;
	goto close;
    }
    /* identify magic header */
    if (((unsigned short *)head)[0] == GZIP_MAGIC) {
	if (print)
	    printf("%s:\t", name);

	/* identify compression method */
	if (head[2] == DEFLATED) {
	    if (print)
		printf("deflated ");
	}
	/* identify flags */
	if (head[3] & ORIGNAME) {
	    char *p;

	    p = origname = xmalloc(FILENAME_MAX + 1);
	    while ((*p++ = fgetc(fp)) != '\0');
	    if (print)
		printf("`%s' ", origname);
	}
	/* identify extflags */
	if (print) {
	    switch (head[8]) {
		case 0x02:
		    printf("max-compression ");
		    break;
		case 0x04:
		    printf("max-speed ");
		    break;
		default:
		    ;
	    }
	}
	/* fix original time stamp */
#ifdef WORDS_BIGENDIAN
	time_stamp = ((((ulong *) head)[1] & 0xFF) << 24) + ((((ulong *) head)[1] & 0xFF00) << 8)
	    + ((((ulong *) head)[1] & 0xFF0000) >> 8) + ((((ulong *) head)[1] & 0xFF000000) >> 24);
#else /* DAMNED Intel!! */
	time_stamp = ((time_t *) head)[1];
#endif

	if (print) {
	    struct tm *tm;
	    char buf[256];

	    buf[0] = '\n';
	    buf[1] = '\0';
	    if (time_stamp) {
		tm = localtime(&time_stamp);
		sprintf(buf, "%s", asctime(tm));
	    }
	    if (*os_name[head[9]] != '\0') {
		sprintf(buf + strlen(buf) - 1 /* \n */ , "(%s)", os_name[head[9]]);
	    }
	    printf("%s\n", buf);
	}
	retcode = TRUE;
    } else if (verbose) {
	printf("%s: %s: unknown suffix -- ignored\n", progname, name);
	retcode = FALSE;
    }
  close:
    fclose(fp);
    return retcode;
}

void 
ztouch (fname)
char *fname;
{
    struct stat st;
    char *newname = fname;
    char name[FILENAME_MAX];

    stat(newname, &st);

    if (!noname && origname != NULL) {
	sprintf(name, "%s%s.gz", dirname(fname), origname);
	if (rename(newname, name) != 0) {
	    if (verbose)
		fprintf(stderr, "%s: %s: %s\n", progname, name, strerror(errno));
	    return;
	}
	newname = name;
    }
    if (time_stamp && time_stamp != st.st_mtime) {
#ifdef HAVE_UTIMES
	struct timeval tvp[2];

	tvp[0].tv_sec = st.st_atime;
	tvp[0].tv_usec = 0;
	tvp[1].tv_sec = time_stamp;
	tvp[1].tv_usec = 0;
	if (utimes(newname, tvp) == 0) {
#else
#ifdef HAVE_UTIME
	struct utimbuf ut;

	ut.actime = st.st_atime;
	ut.modtime = time_stamp;
	if (utime(newname, &ut) == 0) {
#endif
#endif
	    if (verbose)
		printf("%s: %s: time stamp restored\n", progname, fname);
	}
    } else {
	if (verbose)
	    printf("%s: %s: time stamp unchanged\n", progname, fname);
    }
}

void
print_version()
{
    fprintf(stderr,
#if !__STDC__
	    "%s %s (%s)\n", progname, VERSION, REVDATE
#else
	    "%s " VERSION " (" REVDATE ")\n", progname
#endif
    );
}

__volatile void
usage()
{
    if (!version)
	print_version();
    fprintf(stderr, "usage: %s [-hnpvV] [file ...]\n", progname);
    if (help) {
	fprintf(stderr,
		"\
 -h --help        give this help\n\
 -n --no-name     unchange to original name\n\
 -p --print       print info\n\
 -v --verbose     verbose mode\n\
 -V --version     display version number\n"
	    );
    }
    exit(1);
}

void
main(int argc, char *argv[])
{
    int c, option_index = 0;

#if defined(HUMAN68K)
    if (!isatty(STDOUT))
	setmode(STDOUT, O_BINARY);
    if (!isatty(STDERR))
	setmode(STDERR, O_BINARY);
#endif
    progname = xmalloc(strlen(argv[0]) + 1);
    strcpy(progname, argv[0]);
    progname = basename(progname);

    if (argc == 1) {
	version = TRUE;
	usage();
    }
    while ((c = getopt_long(argc, argv, "hnpvV", long_opt, &option_index)) != EOF) {
	switch (c) {
	    case 0:
		*long_opt[option_index].flag = TRUE;
		break;
	    case 'n':
		noname = TRUE;
		break;
	    case 'p':
		print = TRUE;
		break;
	    case 'v':
		verbose = TRUE;
		break;
	    case 'V':
		version = TRUE;
		break;
	    case 'h':
		help = TRUE;
		/* THROUGH */
	    default:
		usage();
	}
    }

    if (version)
	print_version();
    if (help)
	usage();

    {
	long loop;

	for (loop = optind; loop < argc; loop++)
	    if (ident_gzip(argv[loop]) == TRUE && !print)
		ztouch(argv[loop]);
    }
    exit(0);
}
